@@ -432,6 +432,11 @@
  *    Bug fix for <<small mpfr>> ** <<small Python integer>>.
  *    Compile with Python 3.11.
  *
+ *    2.1.1
+ *    Code cleanup.
+ *    Properly return NOTIMPLEMENTED for unsupported arguments in **.
+ *        Issue 319.
+ *
  ************************************************************************
  *
  * Discussion on sizes of C integer types.
@@ -496,7 +501,7 @@
 
 /* The following global strings are used by gmpy_misc.c. */
 
-char gmpy_version[] = "2.1.0";
+char gmpy_version[] = "2.1.1";
 
 char gmpy_license[] = "\
 The GMPY2 source code is licensed under LGPL 3 or later. The supported \
@@ -904,7 +909,7 @@ static PyMethodDef Pygmpy_methods [] =
 };
 
 static char _gmpy_docs[] =
-"gmpy2 2.1.0 - General Multiple-precision arithmetic for Python\n"
+"gmpy2 2.1.1b1 - General Multiple-precision arithmetic for Python\n"
 "\n"
 "gmpy2 supports several multiple-precision libraries. Integer and\n"
 "rational arithmetic is provided by the GMP library. Real floating-\n"
